@@ -7,7 +7,7 @@ package MarpaX::ESLIF::Grammar::Rule::Properties;
 
 our $AUTHORITY = 'cpan:JDDPAUSE'; # AUTHORITY
 
-our $VERSION = '5.0.6'; # TRIAL VERSION
+our $VERSION = '6.0.1'; # TRIAL VERSION
 
 
 #
@@ -33,7 +33,6 @@ sub new {
         'sequence'                 => $args{sequence},
         'proper'                   => $args{proper},
         'minimum'                  => $args{minimum},
-        'internal'                 => $args{internal},
         'propertyBitSet'           => $args{propertyBitSet},
         'hideseparator'            => $args{hideseparator}
     };
@@ -174,18 +173,6 @@ sub getMinimum {
 }
 
 
-sub isInternal {
-    my ($self) = @_;
-
-    return $self->{internal}
-}
-
-
-sub getInternal {
-    goto &isInternal
-}
-
-
 sub getPropertyBitSet {
     my ($self) = @_;
 
@@ -216,7 +203,7 @@ MarpaX::ESLIF::Grammar::Rule::Properties - ESLIF Grammar Rule Properties
 
 =head1 VERSION
 
-version 5.0.6
+version 6.0.1
 
 =head1 SYNOPSIS
 
@@ -353,16 +340,6 @@ Alias to isProper()
 
 Returns Rule's minimum number of RHS (meaningful only when rule is a sequence)
 
-=head2 $self->isInternal
-
-Returns Rule is internal ? (possible only when there is the loosen operator "||")
-
-=head2 $self->getInternal
-
-Returns Rule is internal ? (possible only when there is the loosen operator "||")
-
-Alias to isInternal()
-
 =head2 $self->getPropertyBitSet
 
 Returns Rule's low-level property bits (combination of MarpaX::ESLIF::Grammar::Rule::PropertyBitSet values)
