@@ -7,7 +7,7 @@ package MarpaX::ESLIF::Grammar::Symbol::Properties;
 
 our $AUTHORITY = 'cpan:JDDPAUSE'; # AUTHORITY
 
-our $VERSION = '5.0.6'; # TRIAL VERSION
+our $VERSION = '6.0.1'; # TRIAL VERSION
 
 
 #
@@ -43,7 +43,8 @@ sub new {
         'propertyBitSet'             => $args{propertyBitSet},
         'eventBitSet'                => $args{eventBitSet},
         'symbolAction'               => $args{symbolAction},
-        'ifAction'                   => $args{ifAction}
+        'ifAction'                   => $args{ifAction},
+        'verbose'                    => $args{verbose}
     };
 
     return bless $self, $pkg
@@ -293,6 +294,18 @@ sub getIfAction {
     return $self->{ifAction}
 }
 
+
+sub isVerbose {
+    my ($self) = @_;
+
+    return $self->{verbose}
+}
+
+
+sub getVerbose {
+    goto &isVerbose
+}
+
 __END__
 
 =pod
@@ -305,7 +318,7 @@ MarpaX::ESLIF::Grammar::Symbol::Properties - ESLIF Grammar Symbol Properties
 
 =head1 VERSION
 
-version 5.0.6
+version 6.0.1
 
 =head1 SYNOPSIS
 
@@ -528,6 +541,16 @@ Returns the symbol specific action, null if there is none
 
 Returns the symbol if action, null if there is none
 
+=head2 $self->isVerbose
+
+Returns if the symbol is verbose
+
+=head2 $self->getVerbose
+
+Returns if the symbol is verbose
+
+Alias to isVerbose()
+
 =head1 DESCRIPTION
 
 ESLIF Grammar Symbol Properties.
