@@ -23,6 +23,8 @@
 #define G1_RULE_STATEMENT_19                     "G1_rule_statement_19"
 #define G1_RULE_STATEMENT_20                     "G1_rule_statement_20"
 #define G1_RULE_START_RULE                       "G1_rule_start_rule"
+#define G1_RULE_START_SYMBOL_1                   "G1_rule_start_symbol_1"
+#define G1_RULE_START_SYMBOL_2                   "G1_rule_start_symbol_2"
 #define G1_RULE_DESC_RULE                        "G1_rule_desc_rule"
 #define G1_RULE_EMPTY_RULE                       "G1_rule_empty_rule"
 #define G1_RULE_NULL_STATEMENT                   "G1_rule_null_statement"
@@ -75,6 +77,7 @@
 #define G1_RULE_ADVERB_ITEM_20                   "G1_rule_adverb_item_20"
 #define G1_RULE_ADVERB_ITEM_21                   "G1_rule_adverb_item_21"
 #define G1_RULE_ADVERB_ITEM_22                   "G1_rule_adverb_item_22"
+#define G1_RULE_ADVERB_ITEM_23                   "G1_rule_adverb_item_23"
 #define G1_RULE_ACTION_1                         "G1_rule_action_1"
 #define G1_RULE_ACTION_2                         "G1_rule_action_2"
 #define G1_RULE_ACTION_3                         "G1_rule_action_3"
@@ -105,6 +108,8 @@
 #define G1_RULE_EVENT_INITIALIZER_2              "G1_rule_event_initializer_2"
 #define G1_RULE_LATM_SPECIFICATION_1             "G1_rule_latm_specification_1"
 #define G1_RULE_LATM_SPECIFICATION_2             "G1_rule_latm_specification_2"
+#define G1_RULE_VERBOSE_SPECIFICATION_1          "G1_rule_verbose_specification_1"
+#define G1_RULE_VERBOSE_SPECIFICATION_2          "G1_rule_verbose_specification_2"
 #define G1_RULE_NAMING                           "G1_rule_naming"
 #define G1_RULE_NULL_ADVERB                      "G1_rule_null_adverb"
 #define G1_RULE_ALTERNATIVE_NAME_1               "G1_rule_alternative_name_1"
@@ -114,7 +119,8 @@
 #define G1_RULE_EVENT_NAME_3                     "G1_rule_event_name_3"
 #define G1_RULE_EVENT_NAME_4                     "G1_rule_event_name_4"
 #define G1_RULE_EVENT_NAME_5                     "G1_rule_event_name_4"
-#define G1_RULE_LHS                              "G1_rule_lhs"
+#define G1_RULE_LHS_1                            "G1_rule_lhs_1"
+#define G1_RULE_LHS_2                            "G1_rule_lhs_2"
 #define G1_RULE_RHS                              "G1_rule_rhs"
 #define G1_RULE_RHS_ALTERNATIVE_1                "G1_rule_rhs_alternative_1"
 #define G1_RULE_RHS_ALTERNATIVE_2                "G1_rule_rhs_alternative_2"
@@ -123,8 +129,14 @@
 #define G1_RULE_RHS_ALTERNATIVE_5                "G1_rule_rhs_alternative_5"
 #define G1_RULE_RHS_ALTERNATIVE_6                "G1_rule_rhs_alternative_6"
 #define G1_RULE_RHS_ALTERNATIVE_7                "G1_rule_rhs_alternative_7"
+#define G1_RULE_RHS_ALTERNATIVE_8                "G1_rule_rhs_alternative_8"
+#define G1_RULE_RHS_ALTERNATIVE_9                "G1_rule_rhs_alternative_9"
+#define G1_RULE_RHS_ALTERNATIVE_10               "G1_rule_rhs_alternative_10"
+#define G1_RULE_RHS_PRIMARY_NO_PARAMETER_1       "G1_rule_rhs_primary_no_parameter_1"
+#define G1_RULE_RHS_PRIMARY_NO_PARAMETER_2       "G1_rule_rhs_primary_no_parameter_2"
 #define G1_RULE_RHS_PRIMARY_1                    "G1_rule_rhs_primary_1"
 #define G1_RULE_RHS_PRIMARY_2                    "G1_rule_rhs_primary_2"
+#define G1_RULE_RHS_PRIMARY_3                    "G1_rule_rhs_primary_3"
 #define G1_RULE_SINGLE_SYMBOL_1                  "G1_rule_single_symbol_1"
 #define G1_RULE_SINGLE_SYMBOL_2                  "G1_rule_single_symbol_2"
 #define G1_RULE_TERMINAL_1                       "G1_rule_terminal_1"
@@ -132,6 +144,7 @@
 #define G1_RULE_TERMINAL_3                       "G1_rule_terminal_3"
 #define G1_RULE_TERMINAL_4                       "G1_rule_terminal_4"
 #define G1_RULE_TERMINAL_5                       "G1_rule_terminal_5"
+#define G1_RULE_TERMINAL_6                       "G1_rule_terminal_6"
 #define G1_RULE_SYMBOL                           "G1_rule_symbol"
 #define G1_RULE_SYMBOL_NAME_1                    "G1_rule_symbol_name_1"
 #define G1_RULE_SYMBOL_NAME_2                    "G1_rule_symbol_name_2"
@@ -149,6 +162,7 @@
 #define G1_RULE_ACTION_NAME_12                   "G1_rule_action_name_12"
 #define G1_RULE_ACTION_NAME_13                   "G1_rule_action_name_13"
 #define G1_RULE_ACTION_NAME_14                   "G1_rule_action_name_14"
+#define G1_RULE_ACTION_NAME_15                   "G1_rule_action_name_15"
 #define G1_RULE_SYMBOLACTION_NAME_1              "G1_rule_symbolaction_name_1"
 #define G1_RULE_SYMBOLACTION_NAME_2              "G1_rule_symbolaction_name_2"
 #define G1_RULE_SYMBOLACTION_NAME_3              "G1_rule_symbolaction_name_3"
@@ -159,6 +173,8 @@
 #define G1_RULE_SYMBOLACTION_NAME_8              "G1_rule_symbolaction_name_8"
 #define G1_RULE_SYMBOLACTION_NAME_9              "G1_rule_symbolaction_name_9"
 #define G1_RULE_SYMBOLACTION_NAME_10             "G1_rule_symbolaction_name_10"
+#define G1_RULE_SYMBOLACTION_NAME_11             "G1_rule_symbolaction_name_11"
+#define G1_RULE_SYMBOLACTION_NAME_12             "G1_rule_symbolaction_name_12"
 #define G1_RULE_QUANTIFIER_1                     "G1_rule_quantifier_1"
 #define G1_RULE_QUANTIFIER_2                     "G1_rule_quantifier_2"
 #define G1_RULE_SIGNED_INTEGER                   "G1_rule_signed_integer"
@@ -184,16 +200,29 @@
 #define G1_RULE_IF_ACTION                        "G1_rule_if_action"
 #define G1_RULE_IFACTION_NAME_1                  "G1_rule_ifaction_name_1"
 #define G1_RULE_IFACTION_NAME_2                  "G1_rule_ifaction_name_2"
+#define G1_RULE_IFACTION_NAME_3                  "G1_rule_ifaction_name_3"
+#define G1_RULE_GENERATORACTION_NAME_1           "G1_rule_generatoraction_name_1"
+#define G1_RULE_GENERATORACTION_NAME_2           "G1_rule_generatoraction_name_2"
+#define G1_RULE_GENERATORACTION_NAME_3           "G1_rule_generatoraction_name_3"
 #define G1_RULE_REGEX_ACTION                     "G1_rule_regex_action"
 #define G1_RULE_REGEXACTION_NAME_1               "G1_rule_regexaction_name_1"
 #define G1_RULE_REGEXACTION_NAME_2               "G1_rule_regexaction_name_2"
+#define G1_RULE_REGEXACTION_NAME_3               "G1_rule_regexaction_name_3"
+#define G1_RULE_GENERATOR_ACTION                 "G1_rule_generator_action"
+#define G1_RULE_GENERATORACTION_NAME_1           "G1_rule_generatoraction_name_1"
+#define G1_RULE_GENERATORACTION_NAME_2           "G1_rule_generatoraction_name_2"
+#define G1_RULE_GENERATORACTION_NAME_3           "G1_rule_generatoraction_name_3"
 #define G1_RULE_EVENT_ACTION                     "G1_rule_event_action"
 #define G1_RULE_EVENTACTION_NAME_1               "G1_rule_eventaction_name_1"
 #define G1_RULE_EVENTACTION_NAME_2               "G1_rule_eventaction_name_2"
+#define G1_RULE_EVENTACTION_NAME_3               "G1_rule_eventaction_name_3"
 #define G1_RULE_DEFAULT_ENCODING                 "G1_rule_default_encoding"
 #define G1_RULE_DEFAULTENCODING_NAME             "G1_rule_defaultencoding_name"
 #define G1_RULE_FALLBACK_ENCODING                "G1_rule_fallback_encoding"
 #define G1_RULE_FALLBACKENCODING_NAME            "G1_rule_fallbackencoding_name"
+#define G1_RULE_LUA_FUNCTION                     "G1_rule_lua_function"
+#define G1_RULE_LUA_FUNCTIONCALL                 "G1_rule_lua_functioncall"
+#define G1_RULE_LUA_FUNCTIONDECL                 "G1_rule_lua_functiondecl"
 
 #define G1_ACTION_STATEMENTS                       "::undef"
 #define G1_ACTION_STATEMENT_01                     "::undef"
@@ -217,6 +246,8 @@
 #define G1_ACTION_STATEMENT_19                     "::undef"
 #define G1_ACTION_STATEMENT_20                     "::undef"
 #define G1_ACTION_START_RULE                       "G1_action_start_rule"                       /* done */
+#define G1_ACTION_START_SYMBOL_1                   "G1_action_start_symbol_1"                   /* done */
+#define G1_ACTION_START_SYMBOL_2                   "G1_action_start_symbol_2"                   /* done */
 #define G1_ACTION_DESC_RULE                        "G1_action_desc_rule"                        /* done */
 #define G1_ACTION_EMPTY_RULE                       "G1_action_empty_rule"                       /* done */
 #define G1_ACTION_NULL_STATEMENT                   "::undef"
@@ -268,6 +299,7 @@
 #define G1_ACTION_ADVERB_ITEM_20                   "::shift" /* <adverb item> ::= <default encoding> */
 #define G1_ACTION_ADVERB_ITEM_21                   "::shift" /* <adverb item> ::= <fallback encoding> */
 #define G1_ACTION_ADVERB_ITEM_22                   "::shift" /* <adverb item> ::= <regex action> */
+#define G1_ACTION_ADVERB_ITEM_23                   "::shift" /* <adverb item> ::= <verbose specification> */
 #define G1_ACTION_ACTION_1                         "G1_action_action_1"                         /* done */
 #define G1_ACTION_ACTION_2                         "G1_action_action_2"                         /* done */
 #define G1_ACTION_ACTION_3                         "G1_action_action_3"                         /* done */
@@ -298,6 +330,8 @@
 #define G1_ACTION_EVENT_INITIALIZER_2              "G1_action_event_initializer_2"              /* done */
 #define G1_ACTION_LATM_SPECIFICATION_1             "G1_action_latm_specification_1"             /* done */
 #define G1_ACTION_LATM_SPECIFICATION_2             "G1_action_latm_specification_2"             /* done */
+#define G1_ACTION_VERBOSE_SPECIFICATION_1          "G1_action_verbose_specification_1"           /* done */
+#define G1_ACTION_VERBOSE_SPECIFICATION_2          "G1_action_verbose_specification_2"           /* done */
 #define G1_ACTION_NAMING                           "G1_action_naming"                           /* done */
 #define G1_ACTION_NULL_ADVERB                      "::undef"
 #define G1_ACTION_ALTERNATIVE_NAME_1               "::shift" /* <alternative name> ::= <standard name> */
@@ -307,7 +341,8 @@
 #define G1_ACTION_EVENT_NAME_3                     "::ascii" /* <event name> ::= ':discard[on]' */
 #define G1_ACTION_EVENT_NAME_4                     "::ascii" /* <event name> ::= ':discard[off]' */
 #define G1_ACTION_EVENT_NAME_5                     "::ascii" /* <event name> ::= ':discard[switch]' */
-#define G1_ACTION_LHS                              "::shift" /* lhs ::= <symbol name> */
+#define G1_ACTION_LHS_1                            "G1_action_lhs_1" /* lhs ::= <symbol name> */
+#define G1_ACTION_LHS_2                            "G1_action_lhs_2" /* lhs ::= <lhs> /\-\-?\(/ <lua optional parlist after lparen> */
 #define G1_ACTION_RHS                              "G1_action_rhs"                              /* done */
 #define G1_ACTION_RHS_ALTERNATIVE_1                "G1_action_rhs_alternative_1"                /* done */
 #define G1_ACTION_RHS_ALTERNATIVE_2                "G1_action_rhs_alternative_2"                /* done */
@@ -316,8 +351,14 @@
 #define G1_ACTION_RHS_ALTERNATIVE_5                "G1_action_rhs_alternative_5"                /* done */
 #define G1_ACTION_RHS_ALTERNATIVE_6                "G1_action_rhs_alternative_6"                /* done */
 #define G1_ACTION_RHS_ALTERNATIVE_7                "G1_action_rhs_alternative_7"                /* done */
-#define G1_ACTION_RHS_PRIMARY_1                    "G1_action_rhs_primary_1"                    /* done */
+#define G1_ACTION_RHS_ALTERNATIVE_8                "G1_action_rhs_alternative_8"                /* done */
+#define G1_ACTION_RHS_ALTERNATIVE_9                "G1_action_rhs_alternative_9"                /* done */
+#define G1_ACTION_RHS_ALTERNATIVE_10               "G1_action_rhs_alternative_10"               /* done */
+#define G1_ACTION_RHS_PRIMARY_NO_PARAMETER_1       "G1_action_rhs_primary_no_parameter_1"       /* done */
+#define G1_ACTION_RHS_PRIMARY_NO_PARAMETER_2       "G1_action_rhs_primary_no_parameter_2"       /* done */
+#define G1_ACTION_RHS_PRIMARY_1                    "::shift"                                    /* done */
 #define G1_ACTION_RHS_PRIMARY_2                    "G1_action_rhs_primary_2"                    /* done */
+#define G1_ACTION_RHS_PRIMARY_3                    "G1_action_rhs_primary_3"                    /* done */
 #define G1_ACTION_SINGLE_SYMBOL_1                  "G1_action_single_symbol_1"                  /* done */
 #define G1_ACTION_SINGLE_SYMBOL_2                  "G1_action_single_symbol_2"                  /* done */
 #define G1_ACTION_TERMINAL_1                       "G1_action_terminal_1"                       /* done */
@@ -325,6 +366,7 @@
 #define G1_ACTION_TERMINAL_3                       "G1_action_terminal_3"                       /* done */
 #define G1_ACTION_TERMINAL_4                       "G1_action_terminal_4"                       /* done */
 #define G1_ACTION_TERMINAL_5                       "G1_action_terminal_5"                       /* done */
+#define G1_ACTION_TERMINAL_6                       "G1_action_terminal_6"                       /* done */
 #define G1_ACTION_SYMBOL                           "G1_action_symbol" /* symbol ::= <symbol name> */
 #define G1_ACTION_SYMBOL_NAME_1                    "G1_action_symbol_name_1" /* <symbol name> ::= <bare name> */
 #define G1_ACTION_SYMBOL_NAME_2                    "G1_action_symbol_name_2" /* <symbol name> ::= <bracketed name> */ /* done */
@@ -342,6 +384,7 @@
 #define G1_ACTION_ACTION_NAME_12                   "::ascii" /* <action name> ::= '::row' */
 #define G1_ACTION_ACTION_NAME_13                   "::ascii" /* <action name> ::= '::table' */
 #define G1_ACTION_ACTION_NAME_14                   "::ascii" /* <action name> ::= '::ast' */
+#define G1_ACTION_ACTION_NAME_15                   "::shift" /* <action name> ::= <lua function> */
 #define G1_ACTION_SYMBOLACTION_NAME_1              "::ascii" /* <symbol action name> ::= <ascii graph name> */
 #define G1_ACTION_SYMBOLACTION_NAME_2              "::ascii" /* <symbol action name> ::= '::transfer' */
 #define G1_ACTION_SYMBOLACTION_NAME_3              "::ascii" /* <symbol action name> ::= '::undef' */
@@ -352,6 +395,8 @@
 #define G1_ACTION_SYMBOLACTION_NAME_8              "::ascii" /* <symbol action name> ::= '::true' */
 #define G1_ACTION_SYMBOLACTION_NAME_9              "::ascii" /* <symbol action name> ::= '::false' */
 #define G1_ACTION_SYMBOLACTION_NAME_10             "::ascii" /* <symbol action name> ::= '::json' */
+#define G1_ACTION_SYMBOLACTION_NAME_11             "::ascii" /* <symbol action name> ::= '::jsonf' */
+#define G1_ACTION_SYMBOLACTION_NAME_12             "::shift" /* <symbol action name> ::= <lua function> */
 #define G1_ACTION_QUANTIFIER_1                     "G1_action_quantifier_1"                     /* done */
 #define G1_ACTION_QUANTIFIER_2                     "G1_action_quantifier_2"                     /* done */
 #define G1_ACTION_SIGNED_INTEGER                   "::ascii" /* <signed integer> ::= /[+-]?\d+/ */
@@ -373,23 +418,39 @@
 #define G1_ACTION_LUASCRIPT_SOURCE                 "::concat"
 #define G1_ACTION_IFACTION_NAME_1                  "::ascii" /* <if action name> ::= <ascii graph name> */
 #define G1_ACTION_IFACTION_NAME_2                  "::ascii" /* <if action name> ::= <lua action name> */
+#define G1_ACTION_IFACTION_NAME_3                  "::shift" /* <if action name> ::= <lua function> */
 #define G1_ACTION_IFACTION                         "G1_action_ifaction"                         /* done */
+#define G1_ACTION_GENERATORACTION_NAME_1           "::ascii" /* <generator action name> ::= <ascii graph name> */
+#define G1_ACTION_GENERATORACTION_NAME_2           "::ascii" /* <generator action name> ::= <lua action name> */
+#define G1_ACTION_GENERATORACTION_NAME_3           "::shift" /* <generator action name> ::= <lua function> */
+#define G1_ACTION_GENERATORACTION                  "G1_action_generatoraction"                         /* done */
 #define G1_ACTION_REGEXACTION_NAME_1               "::ascii" /* <regex action name> ::= <ascii graph name> */
 #define G1_ACTION_REGEXACTION_NAME_2               "::ascii" /* <regex action name> ::= <lua action name> */
+#define G1_ACTION_REGEXACTION_NAME_3               "::shift" /* <regex action name> ::= <lua function> */
 #define G1_ACTION_REGEXACTION                      "G1_action_regexaction"                       /* done */
-#define G1_ACTION_EVENTACTION                      "G1_action_eventaction"                   /* done */
+#define G1_ACTION_GENERATORACTION_NAME_1           "::ascii" /* <generator action name> ::= <ascii graph name> */
+#define G1_ACTION_GENERATORACTION_NAME_2           "::ascii" /* <generator action name> ::= <lua action name> */
+#define G1_ACTION_GENERATORACTION_NAME_3           "::shift" /* <generator action name> ::= <lua function> */
+#define G1_ACTION_GENERATORACTION                  "G1_action_generatoraction"                       /* done */
 #define G1_ACTION_EVENTACTION_NAME_1               "::ascii" /* <event action name> ::= <ascii graph name> */
 #define G1_ACTION_EVENTACTION_NAME_2               "::ascii" /* <event action name> ::= <lua identifier> */
+#define G1_ACTION_EVENTACTION_NAME_3               "::shift" /* <event action name> ::= <lua function> */
+#define G1_ACTION_EVENTACTION                      "G1_action_eventaction"                   /* done */
 #define G1_ACTION_DEFAULTENCODING                  "G1_action_defaultencoding"                   /* done */
 #define G1_ACTION_DEFAULTENCODING_NAME             "::ascii" /* <default encoding name> ::= <ascii graph name> */
 #define G1_ACTION_FALLBACKENCODING                 "G1_action_fallbackencoding"                   /* done */
 #define G1_ACTION_FALLBACKENCODING_NAME            "::ascii" /* <fallback encoding name> ::= <ascii graph name> */
+#define G1_ACTION_LUA_FUNCTION                     "G1_action_lua_function" /* <lua function>> ::= '::lua->function(' <lua function after lparen>@+2 */
+#define G1_ACTION_LUA_FUNCTIONCALL                 "G1_action_lua_functioncall" /* <lua functioncall>> ::= '->(' <lua args after lparen>@+2 */
+#define G1_ACTION_LUA_FUNCTIONDECL                 "G1_action_lua_functiondecl" /* <lua functiondecl>> ::= '<-(' <lua args after lparen>@+2 */
 
 typedef struct bootstrap_grammar_terminal {
   int                        idi;                 /* Identifier */
   marpaESLIF_terminal_type_t terminalType;        /* Terminal type */
   char                      *modifiers;           /* Modifiers */
   char                      *utf8s;               /* ASCII UTF-8 compatible encoding */
+  char                      *eventBefores;
+  char                      *eventAfters;
   char                      *testFullMatchs;
   char                      *testPartialMatchs;
 } bootstrap_grammar_terminal_t;
@@ -401,6 +462,11 @@ typedef struct bootstrap_grammar_meta {
   short  discardb;             /* Discard symbol ? */
   short  discardonb;           /* :discard[on] event ? */
   short  discardoffb;          /* :discard[off] event ? */
+  short  lazyb;                /* Lazy ? Only ESLIF is using that */
+  int    lookupLevelDeltai;    /* When >= 0, this is a forced lookupLevelDeltai at symbol level */
+  short  verboseb;             /* Forced verbose mode - only for lexemes */
+  char  *eventBefores;         /* Event before - only for lexemes */
+  char  *eventAfters;          /* Event after - only for lexemes */
 } bootstrap_grammar_meta_t;
 
 typedef enum bootstrap_grammar_rule_type {
